@@ -179,7 +179,7 @@ class BaseRoute:
     def matches(self, scope: Scope) -> typing.Tuple[Match, Scope]:
         raise NotImplementedError()  # pragma: no cover
 
-    def url_path_for(self, __name: str, **path_params: typing.Any) -> URLPath:
+    def url_path_for(self, name: str, /, **path_params: typing.Any) -> URLPath:
         raise NotImplementedError()  # pragma: no cover
 
     async def handle(self, scope: Scope, receive: Receive, send: Send) -> None:
@@ -258,12 +258,12 @@ class Route(BaseRoute):
                     return Match.FULL, child_scope
         return Match.NONE, {}
 
-    def url_path_for(self, __name: str, **path_params: typing.Any) -> URLPath:
+    def url_path_for(self, name: str, /, **path_params: typing.Any) -> URLPath:
         seen_params = set(path_params.keys())
         expected_params = set(self.param_convertors.keys())
 
-        if __name != self.name or seen_params != expected_params:
-            raise NoMatchFound(__name, path_params)
+        if name != self.name or seen_params != expected_params:
+            raise NoMatchFound(name, path_params)
 
         path, remaining_params = replace_params(
             self.path_format, self.param_convertors, path_params
@@ -333,12 +333,12 @@ class WebSocketRoute(BaseRoute):
                 return Match.FULL, child_scope
         return Match.NONE, {}
 
-    def url_path_for(self, __name: str, **path_params: typing.Any) -> URLPath:
+    def url_path_for(self, name: str, /, **path_params: typing.Any) -> URLPath:
         seen_params = set(path_params.keys())
         expected_params = set(self.param_convertors.keys())
 
-        if __name != self.name or seen_params != expected_params:
-            raise NoMatchFound(__name, path_params)
+        if name != self.name or seen_params != expected_params:
+            raise NoMatchFound(name, path_params)
 
         path, remaining_params = replace_params(
             self.path_format, self.param_convertors, path_params
@@ -415,8 +415,8 @@ class Mount(BaseRoute):
                 return Match.FULL, child_scope
         return Match.NONE, {}
 
-    def url_path_for(self, __name: str, **path_params: typing.Any) -> URLPath:
-        if self.name is not None and __name == self.name and "path" in path_params:
+    def url_path_for(self, name: str, /, **path_params: typing.Any) -> URLPath:
+        if self.name is not None and name == self.name and "path" in path_params:
             # 'name' matches "<mount_name>".
             path_params["path"] = path_params["path"].lstrip("/")
             path, remaining_params = replace_params(
@@ -424,13 +424,13 @@ class Mount(BaseRoute):
             )
             if not remaining_params:
                 return URLPath(path=path)
-        elif self.name is None or __name.startswith(self.name + ":"):
+        elif self.name is None or name.startswith(self.name + ":"):
             if self.name is None:
                 # No mount name.
-                remaining_name = __name
+                remaining_name = name
             else:
                 # 'name' matches "<mount_name>:<child_name>".
-                remaining_name = __name[len(self.name) + 1 :]
+                remaining_name = name[len(self.name) + 1 :]
             path_kwarg = path_params.get("path")
             path_params["path"] = ""
             path_prefix, remaining_params = replace_params(
@@ -446,7 +446,7 @@ class Mount(BaseRoute):
                     )
                 except NoMatchFound:
                     pass
-        raise NoMatchFound(__name, path_params)
+        raise NoMatchFound(name, path_params)
 
     async def handle(self, scope: Scope, receive: Receive, send: Send) -> None:
         await self.app(scope, receive, send)
@@ -493,8 +493,8 @@ class Host(BaseRoute):
                 return Match.FULL, child_scope
         return Match.NONE, {}
 
-    def url_path_for(self, __name: str, **path_params: typing.Any) -> URLPath:
-        if self.name is not None and __name == self.name and "path" in path_params:
+    def url_path_for(self, name: str, /, **path_params: typing.Any) -> URLPath:
+        if self.name is not None and name == self.name and "path" in path_params:
             # 'name' matches "<mount_name>".
             path = path_params.pop("path")
             host, remaining_params = replace_params(
@@ -502,13 +502,13 @@ class Host(BaseRoute):
             )
             if not remaining_params:
                 return URLPath(path=path, host=host)
-        elif self.name is None or __name.startswith(self.name + ":"):
+        elif self.name is None or name.startswith(self.name + ":"):
             if self.name is None:
                 # No mount name.
-                remaining_name = __name
+                remaining_name = name
             else:
                 # 'name' matches "<mount_name>:<child_name>".
-                remaining_name = __name[len(self.name) + 1 :]
+                remaining_name = name[len(self.name) + 1 :]
             host, remaining_params = replace_params(
                 self.host_format, self.param_convertors, path_params
             )
@@ -518,7 +518,7 @@ class Host(BaseRoute):
                     return URLPath(path=str(url), protocol=url.protocol, host=host)
                 except NoMatchFound:
                     pass
-        raise NoMatchFound(__name, path_params)
+        raise NoMatchFound(name, path_params)
 
     async def handle(self, scope: Scope, receive: Receive, send: Send) -> None:
         await self.app(scope, receive, send)
@@ -652,13 +652,13 @@ class Router:
             response = PlainTextResponse("Not Found", status_code=404)
         await response(scope, receive, send)
 
-    def url_path_for(self, __name: str, **path_params: typing.Any) -> URLPath:
+    def url_path_for(self, name: str, /, **path_params: typing.Any) -> URLPath:
         for route in self.routes:
             try:
-                return route.url_path_for(__name, **path_params)
+                return route.url_path_for(name, **path_params)
             except NoMatchFound:
                 pass
-        raise NoMatchFound(__name, path_params)
+        raise NoMatchFound(name, path_params)
 
     async def startup(self) -> None:
         """
